@@ -186,27 +186,28 @@ class Sequential():
         self.val_accuracy_records =  []
         self.train_cost_fn = None
         self.val_cost_fn = None
+        self.__activation_functions__ = {"ReLU": ReLU(),
+                                         "Leaky_ReLU": Leaky_ReLU(),
+                                         "Sigmoid": Sigmoid(),
+                                         "Softmax": Softmax()}
+        
+        self.__cost_functions__ = {"binary_crossentropy": Binary_CrossEntropy(),
+                                   "categorical_crossentropy": Categorical_CrossEntropy()}
         self.__is_trained__ = False
         self.runtime = 0.0
     
     def add(self, ac_fn, n_in, n_out):
-        activation_functions = {"ReLU": ReLU(),
-                                "Leaky_ReLU": Leaky_ReLU(),
-                                "Sigmoid": Sigmoid(),
-                                "Softmax": Softmax()}
-        
-        for name,function in activation_functions.items():
+        import copy
+        train_activation_functions = copy.deepcopy(self.__activation_functions__)
+        for name,function in train_activation_functions.items():
             if ac_fn == name:
                 self.train_layers.append(Dense(n_in, n_out))
                 self.train_layers.append(function)
                 break
         if self.val_inputs is not None and self.val_targets is not None:
-            activation_functions = {"ReLU": ReLU(),
-                                    "Leaky_ReLU": Leaky_ReLU(),
-                                    "Sigmoid": Sigmoid(),
-                                    "Softmax": Softmax()}
+            val_activation_functions =  copy.deepcopy(self.__activation_functions__)
             
-            for name,function in activation_functions.items():
+            for name,function in val_activation_functions.items():
                 if ac_fn == name:
                     self.val_layers.append(Dense(n_in, n_out))
                     self.val_layers.append(function)
@@ -294,21 +295,20 @@ class Sequential():
         
     def fit(self, cost_fn, epochs = 1000, lr = 0.1, momentum = None, verbose = 1, live_stats = False):
         self.__is_trained__ = True
-        cost_functions = {"binary_crossentropy": Binary_CrossEntropy(),
-                          "categorical_crossentropy": Categorical_CrossEntropy()}
-        
+        import copy
         from tqdm import tqdm
         import matplotlib.pyplot as plt
         import time
         start = time.time()
         
-        for name,function in cost_functions.items():
+        train_cost_functions = copy.deepcopy(self.__cost_functions__)
+        for name,function in train_cost_functions.items():
             if cost_fn == name:
                 self.train_cost_fn = function
                 break
             
         if self.val_inputs is not None and self.val_targets is not None:
-            val_cost_functions = dict(cost_functions)
+            val_cost_functions = copy.deepcopy(self.__cost_functions__)
             for name,function in val_cost_functions.items(): 
                 if cost_fn == name:
                     self.val_cost_fn = function
@@ -419,7 +419,7 @@ class Sequential():
         else:
             raise RuntimeError("model not trained")
             
-    def save_parameters(self):
+    def save_params(self):
         if self.__is_trained__ == True:
             import pickle
             parameters = {"weights": None, "biases": None}
@@ -430,8 +430,8 @@ class Sequential():
                     weights.append(layer.weights)
                     biases.append(layer.biases)
                 else:
-                    weights.append(np.empty((1,1)))
-                    biases.append(np.empty((1,1)))
+                    weights.append(np.array(0.0))
+                    biases.append(np.array(0.0))
             parameters["weights"] = weights
             parameters["biases"] = biases
             with open('parameters.pkl', 'wb') as f:
@@ -440,7 +440,7 @@ class Sequential():
         else:
             raise RuntimeError("model not trained")
         
-    def load_parameters(self, parameters = None):
+    def load_params(self, parameters = None):
         if type(parameters) == dict or parameters == None:
             import pickle
             if parameters == None:
